# sage_setup: distribution = sagemath-repl
r"""
SageMath version and banner info
"""

# ****************************************************************************
#       Copyright (C) 2005 William Stein <wstein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************
import sys

from sage.env import (SAGE_VERSION, SAGE_VERSION_BANNER, SAGE_BANNER)


def version():
    """
    Return the version of Sage.

    OUTPUT: string

    EXAMPLES::

       sage: version()
       'SageMath version ..., Release Date: ...'
    """
    return SAGE_VERSION_BANNER


def banner_text(full=True):
    """
    Text for the Sage banner.

    INPUT:

    - ``full`` -- boolean (default: ``True``)

    OUTPUT:

    A string containing the banner message.

    If option full is ``False``, a simplified plain ASCII banner is
    displayed; if ``True`` the full banner with box art is displayed.

    EXAMPLES::

        sage: print(sage.misc.banner.banner_text(full=True))
        ┌────────────────────────────────────────────────────────────────────┐
        │ SageMath version ...
        sage: print(sage.misc.banner.banner_text(full=False))
        SageMath version ..., Release Date: ...
    """
    if not full:
        return version()

    bars = "─" * 68
    s = []
    a = s.append
    a('┌' + bars + '┐')
    a("\n│ %-66s │\n" % version())
    python_version = sys.version_info[:3]
    a("│ %-66s │\n" % 'Using Python {}.{}.{}. Type "help()" for help.'.format(*python_version))
    a('└' + bars + '┘')
    pre = version_dict()['prerelease']
    try:
        import sage.all
        have_sage_all = True
    except ImportError:
        have_sage_all = False
    if pre or not have_sage_all:
        red_in = '\033[31m'
        red_out = '\033[0m'
        bars2 = bars.replace('─', '━')
        a('\n')
        a(red_in + '┏' + bars2 + '┓' + '\n')
        if pre:
            a("┃ %-66s ┃\n" % 'Warning: this is a prerelease version, and it may be unstable.')
        if not have_sage_all:
            a("┃ %-66s ┃\n" % 'Warning: sage.all is not available; this is a limited REPL.')
        a('┗' + bars2 + '┛' + red_out)
    return ''.join(s)


def banner():
    """
    Print the Sage banner.

    OUTPUT: none

    If the environment variable ``SAGE_BANNER`` is set to ``no``, no
    banner is displayed. If ``SAGE_BANNER`` is set to ``bare``, a
    simplified plain ASCII banner is displayed. Otherwise, the full
    banner with box art is displayed.

    EXAMPLES::

        sage: import sage.misc.banner; sage.misc.banner.SAGE_BANNER = ''
        sage: sage.misc.banner.banner()
        ┌────────────────────────────────────────────────────────────────────┐
        │ SageMath version ..., Release Date: ...                            │
        │ Using Python .... Type "help()" for help.                          │
        ...
    """
    typ = SAGE_BANNER.lower()

    if typ == "no":
        return

    if typ != "bare":
        try:
            print(banner_text(full=True))
            return
        except UnicodeEncodeError:
            pass

    print(banner_text(full=False))


def version_dict():
    """
    A dictionary describing the version of Sage.

    OUTPUT: dictionary with keys 'major', 'minor', 'tiny', 'prerelease'

    This process the Sage version string and produces a dictionary.
    It expects the Sage version to be in one of these forms::

       N.N
       N.N.N
       N.N.N.N
       N.N.str
       N.N.N.str
       N.N.N.N.str

    where 'N' stands for an integer and 'str' stands for a string.
    The first integer is stored under the 'major' key and the second
    integer under 'minor'.  If there is one more integer, it is stored
    under 'tiny'; if there are two more integers, then they are stored
    together as a float N.N under 'tiny'.  If there is a string, then
    the key 'prerelease' returns True.

    For example, if the Sage version is '3.2.1', then the dictionary
    is {'major': 3, 'minor': 2, 'tiny': 1, 'prerelease': False}.
    If the Sage version is '3.2.1.2', then the dictionary is
    {'major': 3, 'minor': 2, 'tiny': 1.200..., 'prerelease': False}.
    If the Sage version is '3.2.alpha0', then the dictionary is
    {'major': 3, 'minor': 2, 'tiny': 0, 'prerelease': True}.

    EXAMPLES::

        sage: from sage.misc.banner import version_dict
        sage: print("SageMath major version is %s" % version_dict()['major'])
        SageMath major version is ...
        sage: version_dict()['major'] == int(sage.version.version.split('.')[0])
        True
    """
    v = SAGE_VERSION.split('.')
    dict = {}
    dict['major'] = int(v[0])
    dict['minor'] = int(v[1])
    dict['tiny'] = 0
    dict['prerelease'] = False
    try:
        int(v[-1])
    except ValueError:  # when last entry is not an integer
        dict['prerelease'] = True
    if (len(v) == 3 and not dict['prerelease']) or len(v) > 3:
        dict['tiny'] = int(v[2])
    try:
        teeny = int(v[3])
        dict['tiny'] += 0.1 * teeny
    except (ValueError, IndexError):
        pass
    return dict


def require_version(major, minor=0, tiny=0, prerelease=False,
                    print_message=False):
    """
    Return ``True`` if Sage version is at least ``major.minor.tiny``.

    INPUT:

    - ``major`` -- integer
    - ``minor`` -- integer (default: 0)
    - ``tiny`` -- float (default: 0)
    - ``prerelease`` -- boolean (default: ``False``)
    - ``print_message`` -- boolean (default: ``False``)

    OUTPUT: ``True`` if ``major.minor.tiny`` is <= version of Sage, ``False``
    otherwise

    For example, if the Sage version number is 3.1.2, then
    require_version(3, 1, 3) will return False, while
    require_version(3, 1, 2) will return True.
    If the Sage version is 3.1.2.alpha0, then
    require_version(3, 1, 1) will return True, while, by default,
    require_version(3, 1, 2) will return False.  Note, though, that
    require_version(3, 1, 2, prerelease=True) will return True:
    if the optional argument prerelease is True, then a prerelease
    version of Sage counts as if it were the released version.

    If optional argument print_message is ``True`` and this function
    is returning False, print a warning message.

    EXAMPLES::

        sage: from sage.misc.banner import require_version
        sage: require_version(2, 1, 3)
        True
        sage: require_version(821, 4)
        False
        sage: require_version(821, 4, print_message=True)
        This code requires at least version 821.4 of SageMath to run correctly.
        You are running version ...
        False
    """
    vers = version_dict()
    prerelease_checked = (prerelease if vers['prerelease'] else True)
    if (vers['major'] > major
        or (vers['major'] == major and vers['minor'] > minor)
        or (vers['major'] == major and vers['minor'] == minor
            and vers['tiny'] > tiny)
        or (vers['major'] == major and vers['minor'] == minor
            and vers['tiny'] == tiny and prerelease_checked)):
        return True
    else:
        if print_message:
            txt = "This code requires at least version {} of SageMath to run correctly."
            print(txt.format(major + 0.1 * minor + 0.01 * tiny))
            print("You are running version {}.".format(SAGE_VERSION))
        return False



# sage_setup: distribution = sagemath-repl
"""
Test the doctesting framework

Many tests (with expected failures or crashes) are run in a
subprocess, those tests can be found in the ``tests/`` subdirectory.

EXAMPLES::

    sage: import signal
    sage: import subprocess
    sage: import time
    sage: from sage.env import SAGE_SRC
    sage: tests_dir = os.path.join(SAGE_SRC, 'sage', 'doctest', 'tests')
    sage: tests_env = dict(os.environ)

Unset :envvar:`TERM` when running doctests, see :issue:`14370`::

    sage: try:
    ....:     del tests_env['TERM']
    ....: except KeyError:
    ....:     pass
    sage: kwds = {'cwd': tests_dir, 'env':tests_env}

Check that :issue:`2235` has been fixed::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:       "--random-seed=0", "--optional=sage", "longtime.rst"], **kwds)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 longtime.rst
    [0 tests, ...s wall]
    ----------------------------------------------------------------------
    All tests passed!
    ----------------------------------------------------------------------
    ...
    0
    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "-l", "longtime.rst"], **kwds)
    Running doctests...
    Doctesting 1 file.
    sage -t --long --warn-long 0.0 --random-seed=0 longtime.rst
    [1 test, ...s wall]
    ----------------------------------------------------------------------
    All tests passed!
    ----------------------------------------------------------------------
    ...
    0

Check handling of tolerances::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "tolerance.rst"], **kwds)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 tolerance.rst
    **********************************************************************
    File "tolerance.rst", line ..., in sage.doctest.tests.tolerance
    Failed example:
        print(":-(")    # abs tol 0.1
    Expected:
        :-)
    Got:
        :-(
    **********************************************************************
    File "tolerance.rst", line ..., in sage.doctest.tests.tolerance
    Failed example:
        print("1.0 2.0 3.0")  # abs tol 0.1
    Expected:
        4.0 5.0
    Got:
        1.0 2.0 3.0
    **********************************************************************
    File "tolerance.rst", line ..., in sage.doctest.tests.tolerance
    Failed example:
        print("Hello")  # abs tol 0.1
    Expected:
        1.0
    Got:
        Hello
    **********************************************************************
    File "tolerance.rst", line ..., in sage.doctest.tests.tolerance
    Failed example:
        print("1.0")  # abs tol 0.1
    Expected:
        Hello
    Got:
        1.0
    **********************************************************************
    File "tolerance.rst", line ..., in sage.doctest.tests.tolerance
    Failed example:
        print("Hello 1.1")  # abs tol 0.1
    Expected:
        Goodbye 1.0
    Got:
        Hello 1.1
    **********************************************************************
    File "tolerance.rst", line ..., in sage.doctest.tests.tolerance
    Failed example:
        print("Hello 1.0")  # rel tol 1e-6
    Expected:
        Goodbye 0.999999
    Got:
        Hello 1.0
    Tolerance exceeded:
        0.999999 vs 1.0, tolerance 2e-6 > 1e-6
    **********************************************************************
    File "tolerance.rst", line ..., in sage.doctest.tests.tolerance
    Failed example:
        print("Hello 1.0")  # rel tol 1e-6
    Expected:
        Hello ...
    Got:
        Hello 1.0
    Note: combining tolerance (# tol) with ellipsis (...) is not supported
    **********************************************************************
    ...
    1

Test the ``--initial`` option::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "-i", "initial.rst"], **kwds)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 initial.rst
    **********************************************************************
    File "initial.rst", line 4, in sage.doctest.tests.initial
    Failed example:
        a = binomiak(10,5)  # random to test that we still get the exception
    Exception raised:
        Traceback (most recent call last):
        ...
        NameError: name 'binomiak' is not defined
    **********************************************************************
    File "initial.rst", line 14, in sage.doctest.tests.initial
    Failed example:
        binomial(10,5)
    Expected:
        255
    Got:
        252
    **********************************************************************
    ...
    ----------------------------------------------------------------------
    sage -t  --warn-long 0.0 --random-seed=0 initial.rst  # 5 doctests failed
    ----------------------------------------------------------------------
    ...
    1

Test the ``--exitfirst`` option::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "--exitfirst", "initial.rst"], **kwds)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 initial.rst
    **********************************************************************
    File "initial.rst", line 4, in sage.doctest.tests.initial
    Failed example:
        a = binomiak(10,5)  # random to test that we still get the exception
    Exception raised:
        Traceback (most recent call last):
        ...
        NameError: name 'binomiak' is not defined
    **********************************************************************
    ...
    ----------------------------------------------------------------------
    sage -t  --warn-long 0.0 --random-seed=0 initial.rst  # 1 doctest failed
    ----------------------------------------------------------------------
    ...
    1

Test a timeout using the ``SAGE_TIMEOUT`` environment variable.  Also set
``CYSIGNALS_CRASH_NDEBUG`` to help ensure the test times out in a timely
manner (:issue:`26912`)::

    sage: from copy import deepcopy
    sage: kwds2 = deepcopy(kwds)
    sage: kwds2['env'].update({'SAGE_TIMEOUT': '1', 'CYSIGNALS_CRASH_NDEBUG': '1'})
    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "99seconds.rst"], **kwds2)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 99seconds.rst
        Timed out
    **********************************************************************
    Tests run before process (pid=...) timed out:
    ...
    ----------------------------------------------------------------------
    sage -t --warn-long 0.0 --random-seed=0 99seconds.rst  # Timed out
    ----------------------------------------------------------------------
    ...
    4

Test handling of ``KeyboardInterrupt`` in doctests::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "keyboardinterrupt.rst"], **kwds)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 keyboardinterrupt.rst
    **********************************************************************
    File "keyboardinterrupt.rst", line 11, in sage.doctest.tests.keyboardinterrupt
    Failed example:
        raise KeyboardInterrupt
    Exception raised:
        Traceback (most recent call last):
        ...
        KeyboardInterrupt
    **********************************************************************
    ...
    ----------------------------------------------------------------------
    sage -t --warn-long 0.0 --random-seed=0 keyboardinterrupt.rst  # 1 doctest failed
    ----------------------------------------------------------------------
    ...
    1

Interrupt the doctester::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "interrupt.rst"], **kwds)
    Running doctests...
    Doctesting 1 file.
    Killing test interrupt.rst
    ----------------------------------------------------------------------
    Doctests interrupted: 0/1 files tested
    ----------------------------------------------------------------------
    ...
    128

Interrupt the doctester (while parallel testing) when a doctest cannot
be interrupted. We also test that passing a ridiculous number of threads
doesn't hurt::

    sage: F = tmp_filename()
    sage: from copy import deepcopy
    sage: kwds2 = deepcopy(kwds)
    sage: kwds2['env']['DOCTEST_TEST_PID_FILE'] = F  # Doctester will write its PID in this file
    sage: subprocess.call(["sage", "-tp", "1000000", "--timeout=120",  # long time
    ....:      "--die_timeout=10", "--optional=sage",
    ....:      "--warn-long", "0", "99seconds.rst", "interrupt_diehard.rst"], **kwds2)
    Running doctests...
    Doctesting 2 files using 1000000 threads...
    Killing test 99seconds.rst
    Killing test interrupt_diehard.rst
    ----------------------------------------------------------------------
    Doctests interrupted: 0/2 files tested
    ----------------------------------------------------------------------
    ...
    128

Even though the doctester master process has exited, the child process
is still alive, but it should be killed automatically
after the ``die_timeout`` given above (10 seconds)::

    sage: # long time
    sage: pid = int(open(F).read())
    sage: time.sleep(2)
    sage: os.kill(pid, signal.SIGQUIT)  # 2 seconds passed => still alive
    sage: time.sleep(8)
    sage: os.kill(pid, signal.SIGQUIT)  # 10 seconds passed => dead  # random
    Traceback (most recent call last):
    ...
    ProcessLookupError: ...

If the child process is dead and removed, the last output should be as above.
However, the child process interrupted its parent process (see
``'interrupt_diehard.rst'``), and became an orphan process. Depending on the
system, an orphan process may eventually become a zombie process instead of
being removed, and then the last output would just be a blank. Hence the ``#
random`` tag.

Test a doctest failing with ``abort()``::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "abort.rst"], **kwds)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 abort.rst
        Killed due to abort
    **********************************************************************
    Tests run before process (pid=...) failed:
    ...
    ------------------------------------------------------------------------
    Unhandled SIGABRT: An abort() occurred.
    This probably occurred because a *compiled* module has a bug
    in it and is not properly wrapped with sig_on(), sig_off().
    Python will now terminate.
    ------------------------------------------------------------------------
    ...
    ----------------------------------------------------------------------
    sage -t --warn-long 0.0 --random-seed=0 abort.rst  # Killed due to abort
    ----------------------------------------------------------------------
    ...
    16

A different kind of crash::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "fail_and_die.rst"], **kwds)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 fail_and_die.rst
    **********************************************************************
    File "fail_and_die.rst", line 5, in sage.doctest.tests.fail_and_die
    Failed example:
        this_gives_a_NameError
    Exception raised:
        Traceback (most recent call last):
        ...
        NameError: name 'this_gives_a_NameError' is not defined
        Killed due to kill signal
    **********************************************************************
    Tests run before process (pid=...) failed:
    ...
    ----------------------------------------------------------------------
    sage -t --warn-long 0.0 --random-seed=0 fail_and_die.rst  # Killed due to kill signal
    ----------------------------------------------------------------------
    ...
    16

Test that ``sig_on_count`` is checked correctly::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "sig_on.rst"], **kwds)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 sig_on.rst
    **********************************************************************
    File "sig_on.rst", line 6, in sage.doctest.tests.sig_on
    Failed example:
        sig_on_count() # check sig_on/off pairings (virtual doctest)
    Expected:
        0
    Got:
        1
    **********************************************************************
    1 item had failures:
       1 of   5 in sage.doctest.tests.sig_on
        [3 tests, 1 failure, ...]
    ----------------------------------------------------------------------
    sage -t --warn-long 0.0 --random-seed=0 sig_on.rst  # 1 doctest failed
    ----------------------------------------------------------------------
    ...
    1

Test logfiles in serial and parallel mode (see :issue:`19271`)::

    sage: t = tmp_filename()
    sage: subprocess.call(["sage", "-t", "--serial", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "--logfile", t, "simple_failure.rst"],
    ....:      stdout=open(os.devnull, "w"), **kwds)
    1
    sage: print(open(t).read())  # long time
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 simple_failure.rst
    **********************************************************************
    File "simple_failure.rst", line 7, in sage.doctest.tests.simple_failure
    Failed example:
        a * b
    Expected:
        20
    Got:
        15
    **********************************************************************
    1 item had failures:
       1 of   5 in sage.doctest.tests.simple_failure
        [4 tests, 1 failure, ...]
    ----------------------------------------------------------------------
    sage -t --warn-long 0.0 --random-seed=0 simple_failure.rst  # 1 doctest failed
    ----------------------------------------------------------------------
    ...

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "--logfile", t, "simple_failure.rst"],
    ....:      stdout=open(os.devnull, "w"), **kwds)
    1
    sage: print(open(t).read())  # long time
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 simple_failure.rst
    **********************************************************************
    File "simple_failure.rst", line 7, in sage.doctest.tests.simple_failure
    Failed example:
        a * b
    Expected:
        20
    Got:
        15
    **********************************************************************
    1 item had failures:
       1 of   5 in sage.doctest.tests.simple_failure
        [4 tests, 1 failure, ...]
    ----------------------------------------------------------------------
    sage -t --warn-long 0.0 --random-seed=0 simple_failure.rst  # 1 doctest failed
    ----------------------------------------------------------------------
    ...

Test the ``--debug`` option::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "--debug", "simple_failure.rst"],
    ....:      stdin=open(os.devnull), **kwds)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 simple_failure.rst
    **********************************************************************
    File "simple_failure.rst", line 7, in sage.doctest.tests.simple_failure
    Failed example:
        a * b
    Expected:
        20
    Got:
        15
    **********************************************************************
    Previously executed commands:
        s...: a = 3
        s...: b = 5
        s...: a + b
        8
    sage:
    <BLANKLINE>
    Returning to doctests...
    **********************************************************************
    1 item had failures:
       1 of   5 in sage.doctest.tests.simple_failure
        [4 tests, 1 failure, ...]
    ----------------------------------------------------------------------
    sage -t --warn-long 0.0 --random-seed=0 simple_failure.rst  # 1 doctest failed
    ----------------------------------------------------------------------
    ...
    1

Test running under gdb, without and with a timeout::

    sage: subprocess.call(["sage", "-t",  "--warn-long", "0",   # long time, optional: gdb
    ....:      "--random-seed=0", "--optional=sage", "--gdb", "1second.rst"],
    ....:      stdin=open(os.devnull), **kwds)
    exec gdb ...
    Running doctests...
    Doctesting 1 file...
    sage -t... 1second.rst...
        [2 tests, ...s wall]
    ----------------------------------------------------------------------
    All tests passed!
    ----------------------------------------------------------------------
    ...
    0

gdb might need a long time to start up, so we allow 30 seconds::

    sage: subprocess.call(["sage", "-t",  "--warn-long", "0",   # long time, optional: gdb
    ....:      "--random-seed=0", "--optional=sage", "--gdb", "-T30", "99seconds.rst"],
    ....:      stdin=open(os.devnull), **kwds)
    exec gdb ...
    Running doctests...
        Timed out
    4

Test the ``--show-skipped`` option::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "--show-skipped", "show_skipped.rst"], **kwds)
    Running doctests ...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 show_skipped.rst
        2 tests not run due to known bugs
        1 gap test not run
        1 long test not run
        1 not tested test not run
        0 tests not run because we ran out of time
        [2 tests, ...s wall]
    ----------------------------------------------------------------------
    All tests passed!
    ----------------------------------------------------------------------
    ...
    0

Optional tests are run correctly::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0", "--long",  # long time
    ....:      "--random-seed=0", "--show-skipped", "--optional=sage,gap", "show_skipped.rst"], **kwds)
    Running doctests ...
    Doctesting 1 file.
    sage -t --long --warn-long 0.0 --random-seed=0 show_skipped.rst
        2 tests not run due to known bugs
        1 not tested test not run
        0 tests not run because we ran out of time
        [4 tests, ...s wall]
    ----------------------------------------------------------------------
    All tests passed!
    ----------------------------------------------------------------------
    ...
    0

    sage: subprocess.call(["sage", "-t", "--warn-long", "0", "--long",  # long time
    ....:      "--random-seed=0", "--show-skipped", "--optional=gAp", "show_skipped.rst"], **kwds)
    Running doctests ...
    Doctesting 1 file.
    sage -t --long --warn-long 0.0 --random-seed=0 show_skipped.rst
        2 tests not run due to known bugs
        1 not tested test not run
        2 sage tests not run
        0 tests not run because we ran out of time
        [2 tests, ...s wall]
    ----------------------------------------------------------------------
    All tests passed!
    ----------------------------------------------------------------------
    ...
    0

Test an invalid value for ``--optional``::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",
    ....:      "--random-seed=0", "--optional=bad-option", "show_skipped.rst"], **kwds)
    Traceback (most recent call last):
    ...
    ValueError: invalid optional tag 'bad-option'
    1

Test ``atexit`` support in the doctesting framework::

    sage: F = tmp_filename()
    sage: os.path.isfile(F)
    True
    sage: from copy import deepcopy
    sage: kwds2 = deepcopy(kwds)
    sage: kwds2['env']['DOCTEST_DELETE_FILE'] = F
    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "atexit.rst"], **kwds2)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 atexit.rst
        [3 tests, ...s wall]
    ----------------------------------------------------------------------
    All tests passed!
    ----------------------------------------------------------------------
    ...
    0
    sage: os.path.isfile(F)  # long time
    False
    sage: try:
    ....:     os.unlink(F)
    ....: except OSError:
    ....:     pass

Test that random tests are reproducible::

    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=0", "--optional=sage", "random_seed.rst"], **kwds)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=0 random_seed.rst
    **********************************************************************
    File "random_seed.rst", line 3, in sage.doctest.tests.random_seed
    Failed example:
        randint(5, 10)
    Expected:
        9
    Got:
        5
    **********************************************************************
    1 item had failures:
       1 of   2 in sage.doctest.tests.random_seed
        [1 test, 1 failure, ...s wall]
    ----------------------------------------------------------------------
    sage -t --warn-long 0.0 --random-seed=0 random_seed.rst  # 1 doctest failed
    ----------------------------------------------------------------------
    ...
    1
    sage: subprocess.call(["sage", "-t", "--warn-long", "0",    # long time
    ....:      "--random-seed=1", "--optional=sage", "random_seed.rst"], **kwds)
    Running doctests...
    Doctesting 1 file.
    sage -t --warn-long 0.0 --random-seed=1 random_seed.rst
        [1 test, ...s wall]
    ----------------------------------------------------------------------
    All tests passed!
    ----------------------------------------------------------------------
    ...
    0
"""
